/**
 * JASMINe
 * Copyright (C) 2007-2010 Bull S.A.S.
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: MuleWrapper.java 7848 2011-03-21 09:56:53Z danesa $
 * --------------------------------------------------------------------------
 */
package org.ow2.jasmine.monitoring.eventswitch.osgi;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.Set;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.management.MBeanServer;
import javax.management.ObjectName;

import org.mule.MuleManager;
import org.mule.config.ConfigurationBuilder;
import org.mule.config.ReaderResource;
import org.mule.config.builders.MuleXmlConfigurationBuilder;
import org.mule.umo.manager.UMOManager;
import org.ow2.jonas.jmx.JmxService;
import org.ow2.jonas.lib.bootstrap.JProp;

/**
 * Wrapper class for Mule.
 */
public class MuleWrapper {

    /**
     * The JOnAS JMX service.
     */
    private JmxService jmxService;

    /**
     * JMX Server.
     */
    private MBeanServer mbeanServer = null;

    /**
     * ObjectName of the Manager.
     */
    private ObjectName objName = null;

    /**
     * Mule starter thread.
     */
    private Thread muleStarterThread = null;

    /**
     * Logger.
     */
    private Logger logger = Logger.getLogger(this.getClass().getName());

    /**
     * Configures Mule, which will automatically launch it.
     *
     * @throws Exception When something wrong happens ...
     */
    public void start() throws Exception {
        this.mbeanServer = jmxService.getJmxServer();

        while (true) {
            ObjectName j2eeServer = null;
            Set<?> j2eeServerNames = this.mbeanServer.queryNames(new ObjectName("*:j2eeType=J2EEServer,*"), null);
            if (j2eeServerNames.size() > 0) {
                j2eeServer = (ObjectName) j2eeServerNames.iterator().next();
            }

            if (j2eeServer == null) {
                throw new Exception("Failed locating the J2EEServer MBean");
            }

            String state = this.mbeanServer.getAttribute(j2eeServer, "state").toString();
            if (state.contains("running")) {
                // Server is running :)
                break;
            } else if (!state.contains("starting")) {
                throw new Exception("Server state is " + state + ", canceling JASMINe EventSwitch instance creation");
            }

            // Server not started, wait a bit and try again
            Thread.sleep(1000);
        }

        if (MuleManager.isInstanciated()) {
            throw new Exception("A Mule instance is already present");
        }

        InputStream inputStream;
        try {
            inputStream = new FileInputStream(new File(JProp.getConfDir(), "eventswitch-config.xml"));
        } catch (FileNotFoundException e) {
            throw new Exception("Cannot read the config file 'eventswitch-config.xml'", e);
        }
        if (inputStream == null) {
            throw new Exception("Cannot read the config file 'eventswitch-config.xml'");
        }

        final String domainName = jmxService.getDomainName();
        final String serverName = jmxService.getJonasServerName();

        ClassLoader lastClassLoader =
            Thread.currentThread().getContextClassLoader();
        Thread.currentThread().setContextClassLoader(getClass().getClassLoader());
        try {
            try {

                final ConfigurationBuilder builder = new MuleXmlConfigurationBuilder();
                final ReaderResource configReader = new ReaderResource("eventswitch", new InputStreamReader(inputStream));

                this.muleStarterThread = new Thread() {
                    @Override
                    public void run() {
                        try {
                            UMOManager umo = builder.configure(new ReaderResource[]{configReader}, null);
                        } catch (Exception e) {
                            MuleWrapper.this.logger.log(Level.SEVERE,
                                "Failed starting mule", e);
                            MuleManager.getInstance().dispose();
                        }
                    }
                };

                this.muleStarterThread.setContextClassLoader(getClass().getClassLoader());
                this.muleStarterThread.start();
            } catch (Exception e) {
                throw new Exception("Failed configuring Mule", e);
            }
        } finally {
            Thread.currentThread().setContextClassLoader(lastClassLoader);
        }
    }

    /**
     * Disposes of Mule.
     */
    public void stop() {
        try {
            this.mbeanServer.unregisterMBean(this.objName);
        } catch (Exception e) {
            Logger.getLogger(this.getClass().getName()).log(Level.SEVERE, "Failed unregistering EventSwitchManager MBean");
        }

        if (this.muleStarterThread != null) {
            this.muleStarterThread.interrupt();
        }

        MuleManager.getInstance().dispose();
    }

    public void setJmxService(final JmxService jmxService) {
        this.jmxService = jmxService;
    }

}
